/**
 * Abbaye des Morts for Parallax Propeller
 * Copyright (c) 2015 Marco Maccaferri
 *
 * TERMS OF USE: MIT License
 */

#ifndef GAME_H_
#define GAME_H_

#include <stdint.h>

#include "gpu.h"
#include "i2c_driver.h"
#include "spi_driver.h"
#include "graphics.h"

#define I2C_SCL         28
#define I2C_SDA         29

#define GPU_TX          11
#define GPU_VSYNC       (1 << 13)

#define JOY_LCH         22
#define JOY_CLK         23
#define JOY_DATA        24

#define JOY_RIGHT       0x01
#define JOY_LEFT        0x02
#define JOY_DOWN        0x04
#define JOY_UP          0x08
#define JOY_START       0x10
#define JOY_SELECT      0x20
#define JOY_B           0x40
#define JOY_A           0x80

#define MIRROR_FLAG     0x02000000

#define INC_0_30  0x4D
#define INC_0_3   0x4D
#define INC_0_5   0x80
#define INC_0_65  0xA7
#define INC_0_75  0xC0
#define INC_0_9   0xE6
#define INC_1_0   0x100
#define INC_1_1   0x11A
#define INC_1_2   0x133
#define INC_1_3   0x14D
#define INC_1_5   0x180
#define INC_1_6   0x19A
#define INC_2_0   0x200
#define INC_2_3   0x24D
#define INC_2_5   0x280
#define INC_3_0   0x300
#define INC_5_0   0x500
#define INC_6_0   0x600
#define INC_7_0   0x700
#define INC_8_0   0x800
#define INC_10_0  0xA00
#define INC_12_0  0xC00
#define INC_13_0  0xD00
#define INC_14_0  0xE00
#define INC_15_0  0xF00
#define INC_16_0  0x1000
#define INC_17_0  0x1100
#define INC_22_0  0x1600
#define INC_23_0  0x1700
#define INC_44_0  0x2C00
#define INC_56_0  0x3800
#define INC_152_0 0x9800
#define INC_175_0 0xAF00
#define INC_256_0 0x10000
#define GRAVITY   0x1E7    // 1.9

#define WALL_FLAG       0x8000

#define TILE099_ID      WALL_FLAG

#define MAP_SIZE        (32 * 22 * 2)

#define LOAD_EEPROM_OFFSET(a)  ((uint32_t)(a) - 0xc0000000 + 0x8000)

#define Yield__() __asm__ volatile( "" ::: "memory" )

extern int32_t _load_start_cog_hsp2ptx_driver[];

extern uint16_t video_ram[24][32];
extern uint32_t sprite_ram[2 * 12];
extern uint32_t mailbox[4];
extern uint32_t sprite_idx;

extern uint8_t * _overlay_start;
extern uint32_t  _state_ovl_table[][2];
extern uint32_t  _room_ovl_table[][2];

struct enem {
    int32_t type;
    int32_t x;
    int32_t y;
    int32_t direction;
    int32_t tile;
    int32_t animation;
    int32_t limleft;
    int32_t limright;
    int32_t speed;
    int32_t fire;
    int32_t adjustx1;
    int32_t adjustx2;
    int32_t adjusty1;
    int32_t adjusty2;
};

struct hero {
    int32_t x; /* = << 8 */
    int32_t y; /* = << 8 */
    int32_t direction;
    int32_t jump; /* 1-Up, 2-Down */
    int32_t height; /* Limit of jump */
    int32_t animation;
    int32_t points[8]; /* Points of collision */
    int32_t ground; /* Pixel where is ground */
    int32_t collision[4]; /* Collisions, in 4 directions */
    int32_t ducking;
    int32_t checkpoint[4];
    int32_t state[2]; /* Vidas y cruces */
    int32_t flags[7];
    int32_t death;
    int32_t push[4]; /* Pulsaciones de teclas */
    int32_t temp;
};

extern uint32_t room[2];
extern uint32_t changeflag;
extern uint32_t counter[3];
extern int32_t proyec[24];

extern struct hero jean;
extern struct enem enemies[7];

int32_t hs_tx(int32_t _DataSamples, int32_t _DataAddress, int32_t _DataCommand, int32_t _Offset);
void    hs_tx_wait();

void begin_sprite_draw();
void draw_sprite(int32_t x, int32_t y, uint32_t tile);
void end_sprite_draw();

int32_t read_gamepad();

void game(int * state);

#endif /* GAME_H_ */
