/**
 * Abbaye des Morts for Parallax Propeller
 * Copyright (c) 2015 Marco Maccaferri
 *
 * TERMS OF USE: MIT License
 */

#include <propeller.h>

#include "game.h"

uint16_t video_ram[24][32];
uint32_t sprite_ram[2 * 12];
uint32_t mailbox[4];
uint32_t sprite_idx = 0;

static int32_t _hs_tx_cog;
static int32_t _hs_tx_param[2];

extern uint16_t room_01_tilemap[];
extern uint16_t room_02_tilemap[];
extern uint16_t room_03_tilemap[];
extern uint16_t room_04_tilemap[];
extern uint16_t room_05_tilemap[];
extern uint16_t room_06_tilemap[];
extern uint16_t room_07_tilemap[];
extern uint16_t room_08_tilemap[];
extern uint16_t room_09_tilemap[];
extern uint16_t room_10_tilemap[];
extern uint16_t room_11_tilemap[];
extern uint16_t room_12_tilemap[];
extern uint16_t room_13_tilemap[];
extern uint16_t room_14_tilemap[];
extern uint16_t room_15_tilemap[];
extern uint16_t room_16_tilemap[];
extern uint16_t room_17_tilemap[];
extern uint16_t room_18_tilemap[];
extern uint16_t room_19_tilemap[];
extern uint16_t room_20_tilemap[];
extern uint16_t room_21_tilemap[];
extern uint16_t room_22_tilemap[];
extern uint16_t room_23_tilemap[];
extern uint16_t room_24_tilemap[];

int32_t hs_tx(int32_t _DataSamples, int32_t _DataAddress, int32_t _DataCommand, int32_t _Offset);
void hs_tx_wait();

void init_sram();
void copy_eeprom_to_sram(int32_t eeprom_addr, int32_t sram_addr, int32_t count);

void startscreen(int * state);
void history(int * state);
void game(int * state);
void gameover(int * state);
void ending(int * state);

int main()
{
    int state;

    // initialization
    OUTA = (1 << JOY_CLK) | (1 << JOY_LCH);
    DIRA = (1 << JOY_CLK) | (1 << JOY_LCH);
    i2c_initialize();
    spi_init();

    state = 0;

    for (;;) {
        if (state != 2) {
            i2c_read_page(LOAD_EEPROM_OFFSET(_state_ovl_table[state][0]), _overlay_start, _state_ovl_table[state][1]);
        }
        switch(state) {
            case 0:
                startscreen(&state);
                break;
            case 1:
                history(&state);
                break;
            case 2:
                init_sram();
                game(&state);
                break;
            case 3:
                gameover(&state);
                break;
            case 4:
                ending(&state);
                break;
        }
    }

    return 0;
}

int32_t hs_tx(int32_t _DataSamples, int32_t _DataAddress, int32_t _DataCommand, int32_t _Offset)
{
    if (_hs_tx_cog) {
        while (!((_hs_tx_param[0] & 0xf) == 15)) {
            Yield__();
        }
    }

    _hs_tx_param[0] = _DataSamples << 18;
    _hs_tx_param[0] = _hs_tx_param[0] | (_DataAddress << 4);
    _hs_tx_param[0] = _hs_tx_param[0] | _DataCommand;
    _hs_tx_param[1] = _Offset;

    _hs_tx_cog = cognew(_load_start_cog_hsp2ptx_driver, (int32_t )&_hs_tx_param[0]) + 1;

    return 0;
}

void hs_tx_wait() {
    if (_hs_tx_cog) {
        while (!((_hs_tx_param[0] & 0xf) == 15)) {
            Yield__();
        }
    }
}

int32_t read_gamepad() {
    int i;
    int32_t nes_bits = 0;

    OUTA &= ~(1 << JOY_LCH);
    OUTA |= (1 << JOY_LCH);

    for (i = 0; i < 8; i++) {
        nes_bits <<= 1;
        nes_bits |= ((INA >> JOY_DATA) & 1);

        OUTA &= ~(1 << JOY_CLK);
        OUTA |= (1 << JOY_CLK);
    }

    nes_bits = (~nes_bits & 0xFF);
    if (nes_bits == 0xFF) {
        nes_bits = 0;
    }

    return nes_bits;
}

static uint32_t eeprom_map[] = {
    LOAD_EEPROM_OFFSET(room_01_tilemap),
    LOAD_EEPROM_OFFSET(room_02_tilemap),
    LOAD_EEPROM_OFFSET(room_03_tilemap),
    LOAD_EEPROM_OFFSET(room_04_tilemap),
    LOAD_EEPROM_OFFSET(room_05_tilemap),
    LOAD_EEPROM_OFFSET(room_06_tilemap),
    LOAD_EEPROM_OFFSET(room_07_tilemap),
    LOAD_EEPROM_OFFSET(room_08_tilemap),
    LOAD_EEPROM_OFFSET(room_09_tilemap),
    LOAD_EEPROM_OFFSET(room_10_tilemap),
    LOAD_EEPROM_OFFSET(room_11_tilemap),
    LOAD_EEPROM_OFFSET(room_12_tilemap),
    LOAD_EEPROM_OFFSET(room_13_tilemap),
    LOAD_EEPROM_OFFSET(room_14_tilemap),
    LOAD_EEPROM_OFFSET(room_15_tilemap),
    LOAD_EEPROM_OFFSET(room_16_tilemap),
    LOAD_EEPROM_OFFSET(room_17_tilemap),
    LOAD_EEPROM_OFFSET(room_18_tilemap),
    LOAD_EEPROM_OFFSET(room_19_tilemap),
    LOAD_EEPROM_OFFSET(room_20_tilemap),
    LOAD_EEPROM_OFFSET(room_21_tilemap),
    LOAD_EEPROM_OFFSET(room_22_tilemap),
    LOAD_EEPROM_OFFSET(room_23_tilemap),
    LOAD_EEPROM_OFFSET(room_24_tilemap),
};

void init_sram() {
    int32_t i, addr;

    addr = 0;
    for (i = 0; i < 24; i++) {
        if (eeprom_map[i] != 0) {
            copy_eeprom_to_sram(eeprom_map[i], addr, MAP_SIZE);
        }
        addr += MAP_SIZE;
    }
}

void copy_eeprom_to_sram(int32_t eeprom_addr, int32_t sram_addr, int32_t count) {
    uint32_t ack = 0;

    OUTA &= ~SPI_CS;
    spi_write_byte(0x02); // WRITE
    spi_write_addr(sram_addr);

    while (count > 0) {
        i2c_start();                                    // Select the device & send address
        ack = i2c_write(0xA0 | I2C_WRITE | ((eeprom_addr & 0x10000) >> 13));
        ack = (ack << 1) | i2c_write(eeprom_addr >> 8);
        ack = (ack << 1) | i2c_write(eeprom_addr & 0xFF);

        i2c_start();                                    // Reselect the device for reading
        ack = (ack << 1) | i2c_write(0xA0 | I2C_READ | ((eeprom_addr & 0x10000) >> 13));

        eeprom_addr++;
        while (count > 1 && (eeprom_addr & 0xFFFF) != 0) {
            spi_write_byte(i2c_read(I2C_ACK));
            count--;
            eeprom_addr++;
        }

        spi_write_byte(i2c_read(I2C_NAK));
        count--;

        i2c_stop();
    }

    OUTA |= SPI_CS;
}

void begin_sprite_draw()
{
    sprite_idx = 0;
}

void draw_sprite(int32_t x, int32_t y, uint32_t tile)
{
    sprite_ram[sprite_idx++] = ((y & 0x1FF00) << 1) | ((x & 0x1FF00) >> 8);
    sprite_ram[sprite_idx++] = tile;
}

void end_sprite_draw()
{
    while(sprite_idx < 24) {
        sprite_ram[sprite_idx++] = 0;
    }
}
