/**
 * Abbaye des Morts for Parallax Propeller
 * Copyright (c) 2015 Marco Maccaferri
 *
 * TERMS OF USE: MIT License
 */

#include <propeller.h>
#include <stdint.h>

#include "i2c_driver.h"
#include "graphics.h"

#define LOAD_EEPROM_OFFSET(a)  ((uint32_t)(a) - 0xc0000000 + 0x8000)

#define Yield__() __asm__ volatile( "" ::: "memory" )

extern int32_t  _load_start_cog_hsp2prx_driver[];

extern volatile uint32_t vsync;

extern uint32_t tiles_ram[];
extern uint16_t video_ram[24][32];
extern uint32_t sprite_ram[2 * 12];

uint32_t mailbox[4];
uint32_t counter[3];

void start_video_driver();
int32_t hs_rx(int32_t _DataAddress);
void animation();

extern uint32_t intromd_tiles_ram[];
extern uint16_t intromd_tilemap[];

int main()
{
    int cmd;

    i2c_initialize();
    start_video_driver();

    for (;;) {
        cmd = hs_rx(0) & 0x0F;
        switch(cmd) {
            case 0:
                // Do nothing
                break;
            case 1:
                // Counters and animations
                if (counter[0] < 29)
                    counter[0] ++;
                else
                    counter[0] = 0;

                if (counter[1] < 90)
                    counter[1] ++;
                else
                    counter[1] = 0;

                if (counter[2] < 8)
                    counter[2] ++;
                else
                    counter[2] = 0;

                animation();
                break;
            case 2:
                // Load extended tileset
                i2c_read_page(LOAD_EEPROM_OFFSET(mailbox[0]), (uint8_t *)&tiles_ram[OVERLAY_START], mailbox[1]);
                break;
            case 3:
                // Load tileset and map from eeprom
                memset(sprite_ram, 0, sizeof(sprite_ram));
                memset(video_ram, 0, sizeof(video_ram));
                i2c_read_page(LOAD_EEPROM_OFFSET(mailbox[0]), (uint8_t *)tiles_ram, mailbox[1]);
                if (mailbox[2] != 0) {
                    i2c_read_page(LOAD_EEPROM_OFFSET(mailbox[2]), (uint8_t *)video_ram, sizeof(video_ram));
                }
                break;
            case 4:
                // Load map from eeprom
                i2c_read_page(LOAD_EEPROM_OFFSET(mailbox[0]), (uint8_t *)video_ram, sizeof(video_ram));
                break;
            case 5:
                // Restore main tileset
                memset(video_ram, 0, sizeof(video_ram));
                i2c_read_page((uint32_t)tiles_ram, (uint8_t *)tiles_ram, OVERLAY_START * 4);
                break;
        }
    }

    return 0;
}

void start_video_driver()
{
    extern uint32_t _load_start_cogvideo_driver[];
    extern uint32_t _load_start_cogvideo_renderer[];
    extern uint16_t scanline0[];
    extern uint16_t scanline1[];
    extern uint16_t scanline2[];
    extern uint16_t scanline3[];

    cognew(_load_start_cogvideo_driver, &vsync);
    waitcnt(CNT + _CLKFREQ / 1000);

    _load_start_cogvideo_renderer[1] = 0;
#if defined(LCD_BOARD)
    _load_start_cogvideo_renderer[2] = (uint32_t)scanline0;
#else
    _load_start_cogvideo_renderer[2] = (uint32_t)scanline0 + 32;
#endif
    cognew(_load_start_cogvideo_renderer, 0);
    waitcnt(CNT + _CLKFREQ / 1000);

    _load_start_cogvideo_renderer[1] = 1;
#if defined(LCD_BOARD)
    _load_start_cogvideo_renderer[2] = (uint32_t)scanline1;
#else
    _load_start_cogvideo_renderer[2] = (uint32_t)scanline1 + 32;
#endif
    cognew(_load_start_cogvideo_renderer, 0);
    waitcnt(CNT + _CLKFREQ / 1000);

    _load_start_cogvideo_renderer[1] = 2;
#if defined(LCD_BOARD)
    _load_start_cogvideo_renderer[2] = (uint32_t)scanline2;
#else
    _load_start_cogvideo_renderer[2] = (uint32_t)scanline2 + 32;
#endif
    cognew(_load_start_cogvideo_renderer, 0);
    waitcnt(CNT + _CLKFREQ / 1000);

    _load_start_cogvideo_renderer[1] = 3;
#if defined(LCD_BOARD)
    _load_start_cogvideo_renderer[2] = (uint32_t)scanline3;
#else
    _load_start_cogvideo_renderer[2] = (uint32_t)scanline3 + 32;
#endif
    cognew(_load_start_cogvideo_renderer, 0);
    waitcnt(CNT + _CLKFREQ / 1000);
}

int32_t hs_rx(int32_t _DataAddress) {
    int32_t _local__0000[2];

    // Tell PacketHeader where to put the Data it receives
    _local__0000[0] = _DataAddress << 4;
    _local__0000[1] = 0;

    // request data
    cognew(_load_start_cog_hsp2prx_driver, (int32_t )&_local__0000[0]);
    while (!((_local__0000[0] & 0xf) == 15)) {
        Yield__();
    }

    // return feedback data from transmitter
    return _local__0000[1];
}

void animation() {

    int i = 0;
    int j = 0;

    uint32_t data = 0;

    for (j = 0; j <= 21; j++) {
        for (i = 0; i <= 31; i++) {
            data = video_ram[j][i];

            /* Hearts animation */
            if ((data >= TILE401_ID) && (data <= TILE408_ID)) {
                data -= TILE401_ID;
                if (data >= 4) {
                    data -= 4;
                }
                data += TILE401_ID + 4 * (counter[0] / 15);
            }

            /* Cross animation */
            else if ((data >= TILE409_ID) && (data <= TILE416_ID)) {
                data -= TILE409_ID;
                if (data >= 4) {
                    data -= 4;
                }
                data += TILE409_ID + 4 * (counter[0] / 15);
            }

            /* Fire animation */
            else if (data == TILE059_ID || data == TILE060_ID) {
                if ((counter[0] == 1) || (counter[0] == 11) || (counter[0] == 21)) {
                    if (data == TILE059_ID)
                        data = TILE060_ID;
                    else
                        data = TILE059_ID;
                }
            }

            /* Water animation */
            else if ((data >= TILE501_ID) && (data <= TILE504_ID)) {
                if ((counter[0] == 1) || (counter[0] == 11) || (counter[0] == 21)) {
                    if (data < TILE504_ID)
                        data++;
                    else
                        data = TILE501_ID;
                }
            }

            video_ram[j][i] = data;
        }
    }
}
