/**
 * Abbaye des Morts for Parallax Propeller
 * Copyright (c) 2015 Marco Maccaferri
 *
 * TERMS OF USE: MIT License
 */

#include <propeller.h>

#include "spi_driver.h"

void spi_init() {
    OUTA |= SPI_CS | SPI_HOLD;
    DIRA |= SPI_CS | SPI_CLK | SPI_SI | SPI_HOLD;
}

void spi_write_byte(int32_t b) {
    int i;

    for (i = 0; i < 8; i++) {
        if ((b & 0x80) == 0) {
            OUTA &= ~SPI_SI;
        }
        else {
            OUTA |= SPI_SI;
        }
        OUTA |= SPI_CLK;
        OUTA &= ~SPI_CLK;
        b <<= 1;
    }
}

int32_t spi_read_byte() {
    int i;
    int32_t b = 0;

    for (i = 0; i < 8; i++) {
        OUTA |= SPI_CLK;
        b <<= 1;
        if ((INA & SPI_SO) != 0) {
            b |= 1;
        }
        OUTA &= ~SPI_CLK;
    }

    return b;
}

void spi_write_addr(int32_t addr) {
    int i;

    for (i = 0; i < 24; i++) {
        if ((addr & 0x800000) == 0) {
            OUTA &= ~SPI_SI;
        }
        else {
            OUTA |= SPI_SI;
        }
        OUTA |= SPI_CLK;
        OUTA &= ~SPI_CLK;
        addr <<= 1;
    }
}

void spi_write(int32_t addr, const uint8_t * bytes, int32_t length) {
    OUTA &= ~SPI_CS;

    spi_write_byte(0x02); // WRITE
    spi_write_addr(addr);

    while(length > 0) {
        spi_write_byte(*bytes++);
        length--;
    }

    OUTA |= SPI_CS;
}

void spi_read(int32_t addr, uint8_t * bytes, int32_t length) {
    OUTA &= ~SPI_CS;

    spi_write_byte(0x03); // READ
    spi_write_addr(addr);

    while(length > 0) {
        *bytes++ = spi_read_byte();
        length--;
    }

    OUTA |= SPI_CS;
}
